@@ -761,7 +761,8 @@ Pushlibd()
 {
 	char	dirbuf[FILESIZE];
 
-	PathParse(ShareDir, dirbuf);
+	/* Do we even need to make a copy of ShareDir? -- MM */
+	jamstrsub(dirbuf, ShareDir, sizeof(dirbuf));
 	doPushd(dirbuf);
 }
 
@@ -874,12 +875,17 @@ const char	*pre, *post;
  * Note: name and intobuf must not be aliases.
  * Note: because \~ is turned into ~, this routine is not idempotent.
  * ??? I suspect that there are places where in the code that presume
- * it is idempotent!  DHR
+ * it is idempotent!  -- DHR
+ * It seems that all callers of PathParse (or their callers) provide
+ * an intobuf of size FILESIZE, which is important, and it seems
+ * that the operations in PathParse or callees are careful to
+ * not go outside FILESIZE bounds, but I really wish I could feel
+ * more confident of that. -- MM
  */
 void
 PathParse(name, intobuf)
 const char	*name;
-char	*intobuf;
+char	*intobuf;   /* MUST BE at least FILESIZE */
 {
 	char	localbuf[FILESIZE];
 
@@ -891,7 +897,7 @@ char	*intobuf;
 
 	if (*name == '~') {
 		if (name[1] == '/' || name[1] == '\0') {
-			strcpy(localbuf, HomeDir);
+			jamstrsub(localbuf, HomeDir, sizeof(localbuf));
 			name += 1;
 		}
 #ifdef UNIX	/* may add for mac in future */
@@ -917,13 +923,13 @@ char	*intobuf;
 		name += 1;
 #endif /* MSFILESYSTEM */
 	}
-	(void) strcat(localbuf, name);
+	jamstrcat(localbuf, name, sizeof(localbuf));
 
 	/* Make path absolute, and prepare for processing each component
 	 * of the path by placing prefix in intobuf.
 	 */
 #ifndef MSFILESYSTEM
-	strcpy(intobuf, localbuf[0] == '/'? "/" : PWD);
+	jamstrsub(intobuf, localbuf[0] == '/'? "/" : PWD, FILESIZE);
 #else /* MSFILESYSTEM */
 	/* Convert to an absolute path, and then fudge thing so that the
 	 * generic code does not have to deal with drive specifications.
@@ -970,7 +976,7 @@ char	*intobuf;
 				 */
 #ifdef HAS_SYMLINKS
 				char	linkbuf[FILESIZE];
-				int	linklen;
+				JSSIZE_T	linklen;
 #endif
 
 				do {} while (dp > intobuf+1 && *--dp != '/');
@@ -1576,12 +1582,12 @@ private Block
 private daddr	next_bno = 0;
 
 /* Needed to comfort MS Visual C */
-private void blkio proto((Block *, JSSIZE_T (*) ptrproto((int, UnivPtr, size_t))));
+private void blkio proto((Block *, JSSIZE_T (*) ptrproto((int, UnivPtr, JRWSIZE_T))));
 
 private void
 blkio(b, iofcn)
 register Block	*b;
-register JSSIZE_T	(*iofcn) ptrproto((int, UnivPtr, size_t));
+register JSSIZE_T	(*iofcn) ptrproto((int, UnivPtr, JRWSIZE_T));
 {
 	off_t boff = bno_to_seek_off(b->b_bno);
 	JSSIZE_T nb;
@@ -1596,7 +1602,7 @@ register JSSIZE_T	(*iofcn) ptrproto((int, UnivPtr, size_t));
 		      (long)boff, errno, strerror(errno));
 		/* NOTREACHED */
 	}
-	else if ((nb = (*iofcn)(tmpfd, (UnivPtr) b->b_buf, (size_t)JBUFSIZ)) != JBUFSIZ) {
+	else if ((nb = (*iofcn)(tmpfd, (UnivPtr) b->b_buf, (JRWSIZE_T)JBUFSIZ)) != JBUFSIZ) {
 		error("[Tmp file %s error got %D: %d %s: to continue editing would be dangerous]",
 			(iofcn == read) ? "READ" : "WRITE", (long)nb,
 			nb < 0 ? errno : 0, nb < 0 ? strerror(errno): "");
@@ -1652,14 +1658,14 @@ SyncTmp()
 	 */
 	for (bno = 0; bno < next_bno; bno++) {
 		if ((b = lookup_block(bno)) != NULL && b->b_dirty) {
-			blkio(b, (JSSIZE_T (*) ptrproto((int, UnivPtr, size_t)))write);
+			blkio(b, (JSSIZE_T (*) ptrproto((int, UnivPtr, JRWSIZE_T)))write);
 			b->b_dirty = NO;
 		}
 	}
 #else /* !MSDOS */
 	for (b = f_block; b != NULL; b = b->b_LRUnext)
 		if (b->b_dirty) {
-			blkio(b, (JSSIZE_T (*) ptrproto((int, UnivPtr, size_t)))write);
+			blkio(b, (JSSIZE_T (*) ptrproto((int, UnivPtr, JRWSIZE_T)))write);
 			b->b_dirty = NO;
 		}
 #endif /* !MSDOS */
@@ -1724,7 +1730,7 @@ register Block	*bp;
 		bht[B_HASH(bp->b_bno)] = hp->b_HASHnext;
 
 	if (bp->b_dirty) {	/* do, now, the delayed write */
-		blkio(bp, (JSSIZE_T (*) ptrproto((int, UnivPtr, size_t)))write);
+		blkio(bp, (JSSIZE_T (*) ptrproto((int, UnivPtr, JRWSIZE_T)))write);
 		bp->b_dirty = NO;
 	}
 
